/*
 * ReverseFSM.cpp
 *
 *  Created on: 7. 11. 2014
 *	  Author: Tomas Pecka
 */

#include "ReverseFSM.h"

#include <automaton/FSM/MultiInitialStateNFA.h>
#include <exception/AlibException.h>

#include <vector>
#include <algorithm>

namespace automaton {

namespace transform {

automaton::Automaton ReverseFSM::convert(const Automaton& automaton) {
	automaton::Automaton* out = NULL;
	automaton.getData().Accept((void*) &out, ReverseFSM::REVERSE_FSM);
	automaton::Automaton res(std::move(*out));
	delete out;
	return std::move(res);
}

automaton::MultiInitialStateNFA ReverseFSM::convert(const automaton::DFA& automaton)
{
	automaton::MultiInitialStateNFA res;

	res.setStates(automaton.getStates());
	res.addFinalState(automaton.getInitialState());
	for(const auto& q : automaton.getFinalStates())
		res.addInitialState(q);
	res.setInputSymbols(automaton.getInputAlphabet());

	for(const auto& t : automaton.getTransitions())
		res.addTransition(t.second, t.first.second, t.first.first);

	return res;
}

automaton::MultiInitialStateNFA ReverseFSM::convert(const automaton::NFA& automaton)
{
	automaton::MultiInitialStateNFA res;

	res.setStates(automaton.getStates());
	res.addFinalState(automaton.getInitialState());
	for(const auto& q : automaton.getFinalStates())
		res.addInitialState(q);
	res.setInputSymbols(automaton.getInputAlphabet());

	for(const auto& t : automaton.getTransitions())
		for(const auto& q : t.second)
			res.addTransition(q, t.first.second, t.first.first);

	return res;
}

automaton::MultiInitialStateNFA ReverseFSM::convert(const automaton::MultiInitialStateNFA& automaton)
{
	automaton::MultiInitialStateNFA res;

	res.setStates(automaton.getStates());
	for(const auto& q : automaton.getInitialStates())
		res.addFinalState(q);
	for(const auto& q : automaton.getFinalStates())
		res.addInitialState(q);
	res.setInputSymbols(automaton.getInputAlphabet());

	for(const auto& t : automaton.getTransitions())
		for(const auto& q : t.second)
			res.addTransition(q, t.first.second, t.first.first);

	return res;
}


void ReverseFSM::Visit(void* data, const DFA& automaton) const {
	automaton::Automaton*& out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(this->convert(automaton));
}

void ReverseFSM::Visit(void* data, const NFA& automaton) const {
	automaton::Automaton*& out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(this->convert(automaton));
}

void ReverseFSM::Visit(void* data, const MultiInitialStateNFA& automaton) const {
	automaton::Automaton*& out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(this->convert(automaton));
}

void ReverseFSM::Visit(void*, const ExtendedNFA&) const {
	throw exception::AlibException("Unsupported automaton type ExtendedNFA");
}

void ReverseFSM::Visit(void*, const EpsilonNFA&) const {
	throw exception::AlibException("Unsupported automaton type EpsilonNFA");
}

void ReverseFSM::Visit(void*, const CompactNFA&) const {
	throw exception::AlibException("Unsupported automaton type CompactNFA");
}

const ReverseFSM ReverseFSM::REVERSE_FSM;

} /* namespace transform */

} /* namespace automaton */

